/*	$Id: coords.c 2007/11/23 18:46:43 mallet $ */

/*
 * Copyright (c) 1997 LAAS/CNRS                       --  Thu Aug  3 2006
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

#include <math.h>

#include "libimages3d.h"


/**********************************************************************
 *
 * Conversion cartesien -> sphrique
 *
 **********************************************************************/

int data_im3d_cart_to_spher (DATA_IM3D*        im3d)


{
  int          i;		/* Pour boucler */
  int          nb_points;
  DATA_PT3D*   point;
  double       x,y,z;

  /* Validit de l'image */
  if (im3d->header.points != POINTS_CALCULES) {
    printf ("Les points de l'image ne sont pas dtermins\n");
    return (ERR_IM3D_POINTS_NON_CALCULES);
  }
  if (im3d->header.coordonnees != COORDONNEES_CARTESIENNES) {
    printf ("L'image n'est pas exprime en coordonnes cartsiennes\n");
    return (ERR_IM3D_TYPE_COORDONNEES);
  }

  /* Conversion des coordonnes des points */
  nb_points = im3d->header.nblig * im3d->header.nbcol;
  point = im3d->points_tab;
  for (i = 0; i < nb_points; i++) {
    if (point->state == GOOD_PT3D) {
      x = point->coord_1;
      y = point->coord_2;
      z = point->coord_3;

      point->coord_1 = sqrt (x*x + y*y + z*z); /* rho */
      point->coord_2 = atan2(y,x); /* theta */
      point->coord_3 = atan2(z, sqrt (x*x + y*y)); /* phi */
    }

    point->coordonnees = COORDONNEES_SPHERIQUES;

    point++;
  }

  im3d->header.coordonnees = COORDONNEES_SPHERIQUES;

  /* Conversion des covariances */
  if (im3d->header.covariances == COVARIANCES_CALCULEES)
    return (ERR_IM3D_FCT_NON_IMPLEMENTEE);

  /* On redtermine les bornes si elles ont t dtermines */
  if (im3d->header.bornes == BORNES_CALCULEES)
    data_im3d_minmax (im3d);


  return (ERR_IM3D_OK);
}

/**********************************************************************
 *
 * Conversion cartesien -> cylindrique
 *
 **********************************************************************/

int data_im3d_cart_to_cyl (DATA_IM3D*        im3d)


{
  int          i;		/* Pour boucler */
  int          nb_points;
  DATA_PT3D*   point;
  double       x,y,z;

  /* Validit de l'image */
  if (im3d->header.points != POINTS_CALCULES) {
    printf ("Les points de l'image ne sont pas dtermins\n");
    return (ERR_IM3D_POINTS_NON_CALCULES);
  }
  if (im3d->header.coordonnees != COORDONNEES_CARTESIENNES) {
    printf ("L'image n'est pas exprime en coordonnes cartsiennes\n");
    return (ERR_IM3D_TYPE_COORDONNEES);
  }

  /* Conversion des coordonnes des points */
  nb_points = im3d->header.nblig * im3d->header.nbcol;
  point = im3d->points_tab;
  for (i = 0; i < nb_points; i++) {
    if (point->state == GOOD_PT3D) {
      x = point->coord_1;
      y = point->coord_2;
      z = point->coord_3;

      point->coord_1 = sqrt (x*x + y*y); /* r */
      point->coord_2 = atan2(y,x); /* theta */
      point->coord_3 = z;		/* Z */
    }
    point->coordonnees = COORDONNEES_CYLINDRIQUES;
    point++;
  }

  im3d->header.coordonnees = COORDONNEES_CYLINDRIQUES;

  /* Conversion des covariances */
  if (im3d->header.covariances == COVARIANCES_CALCULEES)
    return (ERR_IM3D_FCT_NON_IMPLEMENTEE);

  /* On redtermine les bornes si elles ont t dtermines */
  if (im3d->header.bornes == BORNES_CALCULEES)
    data_im3d_minmax (im3d);

  return (ERR_IM3D_OK);
}

/**********************************************************************
 *
 * Conversion sphrique -> cartesien
 *
 **********************************************************************/

int data_im3d_spher_to_cart (DATA_IM3D*        im3d)


{
  int          i;		/* Pour boucler */
  int          nb_points;
  DATA_PT3D*   point;
  double       theta, phi, rho;
  double       cos_phi;

  /* Validit de l'image */
  if (im3d->header.points != POINTS_CALCULES) {
    printf ("Les points de l'image ne sont pas dtermins\n");
    return (ERR_IM3D_POINTS_NON_CALCULES);
  }
  if (im3d->header.coordonnees != COORDONNEES_SPHERIQUES) {
    printf ("L'image n'est pas exprime en coordonnes sphriques\n");
    return (ERR_IM3D_TYPE_COORDONNEES);
  }

  /* Conversion des coordonnes des points */
  nb_points = im3d->header.nblig * im3d->header.nbcol;
  point = im3d->points_tab;
  for (i = 0; i < nb_points; i++) {
    if (point->state == GOOD_PT3D) {
      rho = point->coord_1;
      theta = point->coord_2;
      phi = point->coord_3;

      cos_phi = cos (phi);

      point->coord_1 = rho * cos_phi * cos (theta); /* X */
      point->coord_2 = rho * cos_phi * sin (theta); /* Y */
      point->coord_3 = rho * sin (phi); /* Z */
    }
    point->coordonnees = COORDONNEES_CARTESIENNES;
    point++;
  }

  im3d->header.coordonnees = COORDONNEES_CARTESIENNES;

  /* Conversion des covariances */
  if (im3d->header.covariances == COVARIANCES_CALCULEES)
    return (ERR_IM3D_FCT_NON_IMPLEMENTEE);

  /* On redtermine les bornes si elles ont t dtermines */
  if (im3d->header.bornes == BORNES_CALCULEES)
    data_im3d_minmax (im3d);

  return (ERR_IM3D_OK);
}

/**********************************************************************
 *
 * Conversion sphrique -> cylindrique
 *
 **********************************************************************/

int data_im3d_spher_to_cyl (DATA_IM3D*        im3d)


{
  int          i;		/* Pour boucler */
  int          nb_points;
  DATA_PT3D*   point;
  double       theta, phi, rho;

  /* Validit de l'image */
  if (im3d->header.points != POINTS_CALCULES) {
    printf ("Les points de l'image ne sont pas dtermins\n");
    return (ERR_IM3D_POINTS_NON_CALCULES);
  }
  if (im3d->header.coordonnees != COORDONNEES_SPHERIQUES) {
    printf ("L'image n'est pas exprime en coordonnes sphriques\n");
    return (ERR_IM3D_TYPE_COORDONNEES);
  }

  /* Conversion des coordonnes des points */
  nb_points = im3d->header.nblig * im3d->header.nbcol;
  point = im3d->points_tab;
  for (i = 0; i < nb_points; i++) {
    if (point->state == GOOD_PT3D) {
      rho = point->coord_1;
      theta = point->coord_2;
      phi = point->coord_3;

      point->coord_1 = rho * cos (phi); /* r */
      point->coord_2 = theta;	/* theta */
      point->coord_3 = rho * sin (phi); /* z */
    }
    point->coordonnees = COORDONNEES_CYLINDRIQUES;
    point++;
  }

  im3d->header.coordonnees = COORDONNEES_CYLINDRIQUES;

  /* Conversion des covariances */
  if (im3d->header.covariances == COVARIANCES_CALCULEES)
    return (ERR_IM3D_FCT_NON_IMPLEMENTEE);

  /* On redtermine les bornes si elles ont t dtermines */
  if (im3d->header.bornes == BORNES_CALCULEES)
    data_im3d_minmax (im3d);

  return (ERR_IM3D_OK);
}

/**********************************************************************
 *
 * Conversion cylindrique -> cartesien
 *
 **********************************************************************/

int data_im3d_cyl_to_cart (DATA_IM3D*        im3d)


{
  int          i;		/* Pour boucler */
  int          nb_points;
  DATA_PT3D*   point;
  double       r, theta, z;

  /* Validit de l'image */
  if (im3d->header.points != POINTS_CALCULES) {
    printf ("Les points de l'image ne sont pas dtermins\n");
    return (ERR_IM3D_POINTS_NON_CALCULES);
  }
  if (im3d->header.coordonnees != COORDONNEES_CYLINDRIQUES) {
    printf ("L'image n'est pas exprime en coordonnes cylindriques\n");
    return (ERR_IM3D_TYPE_COORDONNEES);
  }

  /* Conversion des coordonnes des points */
  nb_points = im3d->header.nblig * im3d->header.nbcol;
  point = im3d->points_tab;
  for (i = 0; i < nb_points; i++) {
    if (point->state == GOOD_PT3D) {
      r = point->coord_1;
      theta = point->coord_2;
      z = point->coord_3;

      point->coord_1 = r * cos (theta); /* x */
      point->coord_2 = r * sin (theta);	/* y */
      point->coord_3 = z; /* z */
    }
    point->coordonnees = COORDONNEES_CARTESIENNES;
    point++;
  }

  im3d->header.coordonnees = COORDONNEES_CARTESIENNES;

  /* Conversion des covariances */
  if (im3d->header.covariances == COVARIANCES_CALCULEES)
    return (ERR_IM3D_FCT_NON_IMPLEMENTEE);

  /* On redtermine les bornes si elles ont t dtermines */
  if (im3d->header.bornes == BORNES_CALCULEES)
    data_im3d_minmax (im3d);

  return (ERR_IM3D_OK);
}


/**********************************************************************
 *
 * Conversion cylindrique -> sphrique
 *
 **********************************************************************/

int data_im3d_cyl_to_spher (DATA_IM3D*        im3d)


{
  int          i;		/* Pour boucler */
  int          nb_points;
  DATA_PT3D*   point;
  double       r, theta, z;

  /* Validit de l'image */
  if (im3d->header.points != POINTS_CALCULES) {
    printf ("Les points de l'image ne sont pas dtermins\n");
    return (ERR_IM3D_POINTS_NON_CALCULES);
  }
  if (im3d->header.coordonnees != COORDONNEES_CYLINDRIQUES) {
    printf ("L'image n'est pas exprime en coordonnes cylindriques\n");
    return (ERR_IM3D_TYPE_COORDONNEES);
  }

  /* Conversion des coordonnes des points */
  nb_points = im3d->header.nblig * im3d->header.nbcol;
  point = im3d->points_tab;
  for (i = 0; i < nb_points; i++) {
    if (point->state == GOOD_PT3D) {
      r = point->coord_1;
      theta = point->coord_2;
      z = point->coord_3;

      if (r == 0.0) {		/* On commence par les diffrents cas
				   particuliers */
	if (z > 0.0) {
	  point->coord_1 = z; /* rho */
	  point->coord_2 = theta; /* theta */
	  point->coord_3 = M_PI / 2; /* phi */
	}
	else if (z < 0.0) {
	  point->coord_1 = - z; /* rho */
	  point->coord_2 = theta; /* theta */
	  point->coord_3 = - M_PI / 2; /* phi */
	  /* Rq: dans les 2 cas pdts, theta n'a pas bcp de sens */
	}
	else {			/* r = z = 0.0 */
	  point->coord_1 = 0.0;
	  point->coord_2 = 0.0;
	  point->coord_2 = 0.0;;
	}
      }
      else {
	point->coord_3 = atan2 (z, r); /* phi, forcment diffrent de M_PI / 2.0  */
	point->coord_1 = r / cos (point->coord_3); /* rho */
	point->coord_2 = theta; /* theta */
      }
    }
    point->coordonnees = COORDONNEES_SPHERIQUES;
    point++;
  }

  im3d->header.coordonnees = COORDONNEES_SPHERIQUES;

  /* Conversion des covariances */
  if (im3d->header.covariances == COVARIANCES_CALCULEES)
    return (ERR_IM3D_FCT_NON_IMPLEMENTEE);

  /* On redtermine les bornes si elles ont t dtermines */
  if (im3d->header.bornes == BORNES_CALCULEES)
    data_im3d_minmax (im3d);

  return (ERR_IM3D_OK);
}

