/*	$Id: ptalloc.c 2007/11/23 19:06:31 mallet $ */

/*
 * Copyright (c) 1997 LAAS/CNRS                       --  Thu Aug  3 2006
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

#include <stdlib.h>
#include <string.h>

#include "libimages3d.h"

/******************************************************************
 *
 * Allocation d'un DATA_PT3D
 *
 ******************************************************************/

int
cree_data_pt3d(DATA_PT3D** pt3d,
	       int covar, int normales, int size_local)
{
  DATA_PT3D*     new_pt3d;

  new_pt3d = (DATA_PT3D*) malloc (sizeof (DATA_PT3D));

  if (new_pt3d == NULL) 
    return (ERR_IM3D_MALLOC_PT3D);
  else
    *pt3d = new_pt3d;

  return init_data_pt3d(*pt3d, covar, normales, size_local);
}

/******************************************************************
 *
 * initialisation  '0' d'un DATA_PT3D
 *
 ******************************************************************/

int
init_data_pt3d(DATA_PT3D *new_pt3d,
	       int covar, int normales, int size_local)
{
  /* On initialise tout  zro (!!! Faire un memset ?) */
  new_pt3d->state = BAD_PT3D;
  new_pt3d->repere = NIL_REPERE;
  new_pt3d->coordonnees = NIL_COORDONNEES;
  new_pt3d->normales = NIL_NORMALES;
  new_pt3d->nb_voisins_normales = NIL_TYPE_NORMALES;
  new_pt3d->covariances = NIL_COVARIANCES;
  new_pt3d->local = NIL_INFOS_LOCALES;
  new_pt3d->flag_perso = 0;

  new_pt3d->coord_1 = 0;
  new_pt3d->coord_2 = 0;
  new_pt3d->coord_3 = 0;

  if (covar && alloue_pt3d_covariances(new_pt3d) != ERR_IM3D_OK)
     return ERR_IM3D_MALLOC_PT3D;

  if (normales && alloue_pt3d_normales(new_pt3d) != ERR_IM3D_OK)
     return ERR_IM3D_MALLOC_NORMALES;

  if (size_local &&
      alloue_pt3d_infos_locales(new_pt3d, size_local) != ERR_IM3D_OK)
     return ERR_IM3D_MALLOC_INFOS_LOCALES;

  return ERR_IM3D_OK;
}

/******************************************************************
 *
 * Allocation des covariances d'un DATA_PT3D
 *
 ******************************************************************/

int
alloue_pt3d_covariances(DATA_PT3D*    pt3d)
{
  /* On verifie que le tableau des covariances n'est pas dj allou */
  if (pt3d->covariances != NIL_COVARIANCES)
     free (pt3d->covariances_tab);

  pt3d->covariances_tab
    = (float*) malloc (IM3D_COVARIANCES_SIZE * sizeof(float));

  if (pt3d->covariances_tab == NULL)
    return (ERR_IM3D_MALLOC_PT3D_COVARIANCES);

  /* Mise  jour des paramtres et des flags */
  pt3d->covariances = COVARIANCES_ALLOUEES;

  /* On met tout  zro (intret ?) */
  memset ( (float*) pt3d->covariances_tab, 0, 
	   IM3D_COVARIANCES_SIZE * sizeof(float) );
  

  return (ERR_IM3D_OK);
}

/******************************************************************
 *
 * Allocation des normales d'un DATA_PT3D
 *
 ******************************************************************/

int
alloue_pt3d_normales(DATA_PT3D*    pt3d)
{
  /* On verifie que le tableau des normales n'est pas dj allou */
  if (pt3d->normales != NIL_NORMALES)
     free (pt3d->normales_tab);

  pt3d->normales_tab
    = (float*) malloc (IM3D_NORMALES_SIZE * sizeof(float));

  if (pt3d->normales_tab == NULL)
    return (ERR_IM3D_MALLOC_PT3D_NORMALES);

  /* Mise  jour des paramtres et des flags */
  pt3d->normales = NORMALES_ALLOUEES;

  /* On met tout  zro (intret ?) */
  memset ( (float*) pt3d->normales_tab, 0, 
	   IM3D_NORMALES_SIZE * sizeof(float) );
  
  return (ERR_IM3D_OK);
}

/******************************************************************
 *
 * Allocation du tableau des infos locales d'un DATA_PT3D
 *
 ******************************************************************/

int
alloue_pt3d_infos_locales (DATA_PT3D*    pt3d,
			   int           size_infos)
{
  /* On verifie que le tableau des infos locales n'est pas dj allou */
   if (pt3d->local != NIL_INFOS_LOCALES)
     free (pt3d->local_tab);

  pt3d->local_tab = (void*) malloc (size_infos);

  pt3d->local = INFOS_LOCALES_ALLOUEES;

  if (pt3d->local_tab == NULL)
    return (ERR_IM3D_MALLOC_PT3D_INFOS_LOCALES);

  return (ERR_IM3D_OK);
}

/******************************************************************
 *
 * Destruction d'un POINT3D
 *
 ******************************************************************/

void
del_data_pt3d(DATA_PT3D** pt3d)
{
   empty_data_pt3d(*pt3d);

   /* On efface la structure elle-mme */
   free (*pt3d);
   *pt3d = NULL;			/* Utilit ? */
}

void
empty_data_pt3d(DATA_PT3D *pt3d)
{
   /* On commence par effacer les ventuels tableaux de donnes */
   if (pt3d->covariances != NIL_COVARIANCES)
      free (pt3d->covariances_tab);
   if (pt3d->normales != NIL_NORMALES)
      free (pt3d->normales_tab);
   if (pt3d->local != NIL_INFOS_LOCALES)
      free (pt3d->local_tab);
}

