/*	$Id: ptinfo.c 2007/11/23 18:46:40 mallet $ */

/*
 * Copyright (c) 1997 LAAS/CNRS                       --  Thu Aug  3 2006
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

#include <math.h>

#include "libimages3d.h"

/**********************************************************************
 *
 * Affiche  l'cran tout ce qu'on sait sur le point 3D
 *
 *********************************************************************/

int info_data_pt3d (DATA_PT3D*  pt3d)

{
  /* Le repre des coordonnes du point */
  switch (pt3d->repere) {
  case 0:
    printf ("\tRepre inconnu !!!\n");
    break;
  case 1:
    printf ("\tCoordonnes dans le repre capteur\n");
    break;
  case 2:
    printf ("\tCoordonnes dans un repre diffrent du repre capteur\n");
    break;
  default:
    printf ("\tRepre inconnu !!!\n");
    break;
  }

  /* Les coordonnes du point */
  switch (pt3d->coordonnees) {
  case COORDONNEES_CARTESIENNES:
    printf ("\tPoint exprim en coordonnes cartsiennes :\n");
    printf ("\t\tX = %f\n", pt3d->coord_1);
    printf ("\t\tY = %f\n", pt3d->coord_2);
    printf ("\t\tZ = %f\n", pt3d->coord_3);
    break;
  case COORDONNEES_CYLINDRIQUES:
    printf ("\tPoint exprim en coordonnes cylindriques :\n");
    printf ("\t\tRayon = %f\n", pt3d->coord_1);
    printf ("\t\tTheta = %f\n", pt3d->coord_2 * 180./M_PI);
    printf ("\t\tZ = %f\n", pt3d->coord_3);
    break;
  case COORDONNEES_SPHERIQUES:
    printf ("\tPoint exprim en coordonnes sphriques :\n");
    printf ("\t\tRayon = %f\n", pt3d->coord_1);
    printf ("\t\tTheta = %f\n", pt3d->coord_2 * 180./M_PI);
    printf ("\t\tPhi   = %f\n", pt3d->coord_3 * 180./M_PI);
    break;
  default:
    printf ("\tPoint exprim en coordonnes INCONNUES !!!\n");
    break;
  }

  /* Son vecteur normal... */
  switch (pt3d->normales) {
  case NIL_NORMALES:
    printf ("\t(Vecteur normal non allou)\n");
    break;
  case NORMALES_ALLOUEES:
    printf ("\tVecteur normal simplement allou\n");
    break;
  case NORMALES_CALCULEES:
    printf ("\tVecteur normal ");
    switch (pt3d->nb_voisins_normales) {
    case QUATRE_VOISINS:
      printf ("(calcul sur 4 voisins)");
      break;
    case HUIT_VOISINS:
      printf ("(calcul sur 8 voisins)");
      break;
    default:
      /* On ne devrait jamais arriver l */
      return (ERR_IM3D_CAS_IMPOSSIBLE);
/*       break; */
    }
    printf (" : %.2f %.2f %.2f\n",
	    *(pt3d->normales_tab),
	    *(pt3d->normales_tab + 1),
	    *(pt3d->normales_tab + 2));
    break;
  }

  /* ... ses covariances... */
  switch (pt3d->covariances) {
  case NIL_COVARIANCES:
    printf ("\t(Matrice de covariances non alloue)\n");
    break;
  case COVARIANCES_ALLOUEES:
    printf ("\tMatrice de covariances simplement alloue\n");
    break;
  case COVARIANCES_CALCULEES:
    printf ("\tMatrice de covariances : \n");
    printf ("\t\t%.5f \t%.5f \t%.5f\n",
	    *(pt3d->covariances_tab),
	    *(pt3d->covariances_tab + 1),
	    *(pt3d->covariances_tab + 2));
    printf ("\t\t%.5f \t%.5f \t%.5f\n",
	    *(pt3d->covariances_tab + 1),
	    *(pt3d->covariances_tab + 3),
	    *(pt3d->covariances_tab + 4));
    printf ("\t\t%.5f \t%.5f \t%.5f\n",
	    *(pt3d->covariances_tab + 2),
	    *(pt3d->covariances_tab + 4),
	    *(pt3d->covariances_tab + 5));
    printf ("\n");
    break;
  }

  /* ... et ses infos locales */
  switch (pt3d->local) {
  case NIL_INFOS_LOCALES:
    printf ("\t(Pas d'infos locales alloues)\n");
    break;
  case INFOS_LOCALES_ALLOUEES:
    printf ("\tTableau des infos locales seulement allou\n");
    break;
  case INFOS_LOCALES_CALCULEES:
    printf ("\tTableau des infos locales dtermin\n");
    break;
  }

  return (ERR_IM3D_OK);
}

